"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var createRequire_1 = require("./createRequire");
var getDeps = function (packageJsonPath, requireDep, deps) {
    var pkg = JSON.parse(fs.readFileSync(packageJsonPath, 'utf8'));
    var pkgDeps = Object.keys(pkg.dependencies || {});
    var result = __assign({}, deps);
    for (var _i = 0, pkgDeps_1 = pkgDeps; _i < pkgDeps_1.length; _i++) {
        var dep = pkgDeps_1[_i];
        if (!dep.startsWith('.') && !result[dep]) {
            var depPkg = requireDep.resolve(dep + '/package.json');
            result[dep] = depPkg;
            var subDeps = getDeps(depPkg, requireDep, result);
            result = __assign({}, result, subDeps);
        }
    }
    return result;
};
var entryExts = ['js', 'jsx', 'ts', 'tsx'];
var entryDirs = ['.', 'src'];
var entryCandidates = [];
var _loop_1 = function (dir) {
    entryCandidates = entryCandidates.concat(entryExts.map(function (ext) { return path.join(dir, 'index.' + ext); }));
};
for (var _i = 0, entryDirs_1 = entryDirs; _i < entryDirs_1.length; _i++) {
    var dir = entryDirs_1[_i];
    _loop_1(dir);
}
var isJsApp = function (pkg) {
    var result = false;
    if (pkg.dependencies) {
        var keys = Object.keys(pkg.dependencies);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            if (key.startsWith('@jsapp')) {
                result = true;
            }
        }
    }
    return result;
};
exports.default = function (pkg, packageJsonPath) {
    if (!isJsApp(pkg)) {
        return undefined;
    }
    var requireDep = createRequire_1.default(path.dirname(packageJsonPath));
    var deps = getDeps(packageJsonPath, requireDep, {});
    var stack = [];
    if (deps['body-parser']) {
        stack.push('server');
    }
    if (deps['babel-core']) {
        stack.push('es6');
    }
    stack.push('js');
    if (deps.typescript) {
        stack.push('ts');
    }
    if (deps['apollo-server-express']) {
        stack.push('apollo');
    }
    if (deps.webpack) {
        stack.push('webpack');
    }
    var entry;
    for (var _i = 0, entryCandidates_1 = entryCandidates; _i < entryCandidates_1.length; _i++) {
        var entryPath = entryCandidates_1[_i];
        if (fs.existsSync(entryPath)) {
            entry = entryPath;
            break;
        }
    }
    if (!entry) {
        throw new Error('Cannot find entry file, tried: ' + entryCandidates);
    }
    var builder = {
        entry: entry,
        stack: stack,
        silent: true,
        nodeDebugger: false
    };
    var config = {
        builders: (_a = {},
            _a[pkg.name] = builder,
            _a)
    };
    return config;
    var _a;
};
//# sourceMappingURL=inferConfig.js.map